/*!
  * vue-compose-promise v0.0.0
  * (c) 2019 Eduardo San Martin Morote
  * @license MIT
  */
var VueComposePromise = (function (compositionApi) {
  'use strict';

  function usePromise(options) {
      if (options === void 0) { options = {}; }
      var state = compositionApi.reactive({
          promise: compositionApi.ref(options.promise || null),
          isPending: compositionApi.ref(true),
          data: compositionApi.ref(null),
          error: compositionApi.ref(null),
          isDelayOver: compositionApi.ref(false),
      });
      var timerId = null;
      // reactive to have automatic unwrapping
      var localOptions = compositionApi.reactive({
          pendingDelay: options.pendingDelay == null ? 200 : options.pendingDelay,
      });
      function setupDelay() {
          if (localOptions.pendingDelay > 0) {
              state.isDelayOver = false;
              if (timerId)
                  clearTimeout(timerId);
              timerId = setTimeout(function () { return (state.isDelayOver = true); }, localOptions.pendingDelay);
          }
          else {
              state.isDelayOver = true;
          }
      }
      compositionApi.watch(function () { return state.promise; }, function (newPromise) {
          state.isPending = true;
          state.error = null;
          if (!newPromise) {
              state.data = null;
              state.isDelayOver = false;
              if (timerId)
                  clearTimeout(timerId);
              timerId = null;
              return;
          }
          setupDelay();
          newPromise
              .then(function (value) {
              // ensure we are dealing with the same promise
              if (state.promise === newPromise) {
                  // @ts-ignore seems to be a bug in @vue/composition-api
                  state.data = value;
                  state.isPending = false;
              }
          })
              .catch(function (err) {
              // ensure we are dealing with the same promise
              if (state.promise === newPromise) {
                  state.error = err;
                  state.isPending = false;
              }
          });
      });
      return {
          state: state,
          options: localOptions,
          set: function (p) { return (state.promise = p); },
      };
  }

  return usePromise;

}(compositionApi));
